/***************************************************************************
   sac2local.cpp 
      Transform sac files to local format (pc to unix or unix to pc).

    begin                : 10/2002
    copyright            : by Meijian An
    email                : meijian@iag.usp.br
 ***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#include "Sac_head.h"      /*struct SacHead & sac_null*/

int SacHeadPc2Sun(struct SacHead * pHead);
int Sun2PC( char *cbData, int nSize, int ibyte);
int CheckSacFile(char *cbFileName /*sac file name, can include directory*/, struct SacHead * psacHead /*valid sac head*/= NULL);
void ShowHelp(char *argv0);

int main(int argc, char** argv)
{
	if(argc<2) ShowHelp(argv[0]);
	
	struct SacHead sacHdr;
	FILE *fpwrt, *fprd;
	char cbFileNew[256];
	
	int iUpdate=1,iErr;
	if(strncmp(argv[1],"-d",2)==0) iUpdate=2; //the old file will be updated by local format.
	
	for(int i=iUpdate; i<argc; i++)
	{
		printf("%s",argv[i]);
		iErr=1;
		if(CheckSacFile(argv[i],&sacHdr)==-1)//format of sac file is in other system.
		{
			// "New file name" = lcl."Old Name"
			char *cb;
#ifdef WIN32
			if((cb=strrchr(argv[i],'\\')) == NULL )
#else
			if((cb=strrchr(argv[i],'/')) == NULL )
#endif			
				sprintf(cbFileNew,"lcl.%s",argv[i]);
			else
			{
				strncpy(cbFileNew,argv[i],cb + 1 - argv[i]);
				cbFileNew[cb - argv[i]+1]=0;
				
				strcat(cbFileNew,"lcl.");				
				strcat(cbFileNew,cb+1);				
			}
			
			if( iUpdate==2 ) printf(" => %s ",cbFileNew);
			if( (fprd=fopen(argv[i],"r")) == NULL ) goto nextfile;
			if( (fpwrt=fopen(cbFileNew,"w")) == NULL ) goto nextfile;

			//Write sac file into new file (lcl.*)
			if(fwrite(&sacHdr,1,sizeof(struct SacHead),fpwrt)<sizeof(struct SacHead))
			{
				printf(" \n%s: Error When write sac head !\n",argv[i]);
				goto nextfile;
			}

			if(fseek(fprd, sizeof(struct SacHead), SEEK_SET)!=0)
				goto nextfile;
			
			float ff;
			int j;
			for(j=0;j<sacHdr.npts;j++)
			{
				if(fread(&ff,sizeof(float),1,fprd)<1)
					goto nextfile;
				Sun2PC((char*)&ff, 1, sizeof(float));
				if(fwrite(&ff,sizeof(float),1,fpwrt)<1)
					goto nextfile;
			}
			
			if(iUpdate==1)
			{ 
				remove(argv[i]);
				rename(cbFileNew,argv[i]);
				printf(" Updated!");
			}else
				printf(" Created!");		
			iErr=0;	

nextfile:
			fclose(fprd);
			fclose(fpwrt);
			if(iErr)remove(cbFileNew);
		}
		
		printf("\n");
	}

	return 1;
}

/*
int CheckSacFile(char *cbFileName, struct SacHead * psacHead = NULL)
     return 0: invalid sac file.
            1: is sac file in current system.
           -1: is sac file but is other system (eg. PC sac in SUN)
*/
int CheckSacFile(char *cbFileName /*sac file name, can include directory*/, struct SacHead * psacHead /*valid sac head =NULL*/)
{
	struct stat fst;		
	if(stat(cbFileName, &fst)==-1)
	{
	   perror(cbFileName);
	   return 0;
	}
	
	if((unsigned int)(fst.st_size) <= sizeof(struct SacHead)) return 0; /*file size should be more than siz eof sac header*/
	
	FILE *pf;
	if((pf=fopen(cbFileName,"rb"))==NULL) return FALSE;

	static struct SacHead sacHdr;
	char* pcbsach=(char* )&sacHdr;
	//if(fread(&sacHdr,sizeof(struct SacHead),1,pf) != 1)
	if(fread(pcbsach,1,sizeof(struct SacHead),pf) != sizeof(struct SacHead))
	{
		fclose(pf);
		return 0;
	}	

	fclose(pf);

	int iSacOtherSys=0;
	if(fst.st_size != (int)(sizeof(struct SacHead)+sizeof(float)*sacHdr.npts)) /*file size should be sizeof sac header + data size*/
	{
		if(SacHeadPc2Sun(&sacHdr)==0) return 0;
		if(sacHdr.iftype>6||sacHdr.iftype<1) return 0;   
		if(fst.st_size != (int)(sizeof(struct SacHead)+sizeof(float)*sacHdr.npts))
		   return 0;
		iSacOtherSys = -1;   
	}
	else 
		iSacOtherSys = 1;	

	/*In sac header, there are some values =(-12345)*/
	long * plsac;
	char * pcsac;
	float * pfsac=(float *)(&sacHdr);	
	int i;
	for(i=0;i<70;i++)/*check 70 float varibles*/
	{
		if(*(pfsac+i)==(float)(-12345))goto checkend;
	}
	
	plsac=(long*)(pfsac+70);
	for(i=0;i<40;i++)/*check 40 long varibles*/
	{
		if(*(plsac+i)==-12345)goto checkend;
	}

	pcsac=(char*)(plsac+40);
	for(i=0;i<24;i++)/*check 24 char[8] varibles*/
	{
		if(!strncmp(pcsac,"-12345  ",8)) goto checkend;
		pcsac+=8;
	}
	
	return 0;

checkend:
	if(psacHead!=NULL)
		memcpy(psacHead, (const void*)&sacHdr, sizeof(struct SacHead));
		
	return iSacOtherSys;
}

/*
   Convert data between Sun (ieee) format and PC format
   modified from ieee2intel in program of R. B. Herrmann
*/
int Sun2PC( char *cbData, int nSize, int ibyte)
/*	cbData:	address of the data.
	nSize:	array size of the data.
	ibyte:	varible byte size. eg.
	
	example: double aa[100];  Sun2PC((char*)aa,100,sizeof(double));
*/
{
	char cb[32];
	int i,j;
	if(ibyte>32){/*SHOW("varible byte is more than 32");*/return 0;}
	for (i=0; i < nSize*ibyte; i+=ibyte )
	{
		for(j=0;j<ibyte;j++)
			cb[j]=cbData[i+j];
		for(j=0;j<ibyte;j++)
			cbData[i+j]=cb[ibyte-j-1];
	}
	return 1;
}

int SacHeadPc2Sun(struct SacHead * pHead)
{
    char *cb=(char *)pHead;

    Sun2PC(cb,70,sizeof(float));
  	Sun2PC(cb+70*sizeof(float),40,sizeof(long));
    return 1;	
}

void ShowHelp(char *argv0)
{
	printf("This program checks the files' formats and only swap sac files to the format for your computer. for example:\n");
	printf("\t%s -d aa/a.sacs b.sac aa.txt  \n\t\t create aa/lcl.a.sacs lcl.b.sac in local format. ascii file aa.txt will be ignored.\n",argv0);
	printf("\t%s * \n\t\t update sac files to local format.\n\n",argv0);
	printf("Suggestion and bug reports are welcome!\n");
	printf("  Released in 12/2002\n");
	printf("     by Meijian An (meijian@iag.usp.br)\n");
	printf("        http://www.geocities.com/meijianan\n");
	exit(0);
}


